-- Enumerates the contents of a folder into a table, printf-able if you add the filetree variable

function recursiveEnumerate(folder, fileTree)
	local fileTree = fileTree or null
    local lfs = love.filesystem
    local filesTable = lfs.enumerate(folder)
	if fileTree == null then
		return filesTable
	else
		for i,v in ipairs(filesTable) do
			local file = folder.."/"..v
			if lfs.isFile(file) then
				fileTree = fileTree.."\n"..file
			elseif lfs.isDirectory(file) then
				fileTree = fileTree.."\n"..file.." (DIR)"
				fileTree = recursiveEnumerate(file, fileTree)
			end
		end
		return filetree
	end
end

-- Y ORDERING

function ysort (a,b)
	return a.y>b.y
end

-- REVERSE

function reverse(item)
	return -item
end

-- ROUNDING TO NEAREST

function roundtointeger(item)
	local integer, fraction = math.modf(item)
	if fraction >= 5 then
		integer = integer + 1
	end
	return integer
end

-- SPLITTING A STRING

function stringsplit(input,iterator)
	iterator = iterator or "."
	local output = {}
	for word in input:gmatch(iterator) do 
		table.insert(output, word)
	end
	return output
end

-- finds matching variables accross a table and puts their positions into a table
function findvar(data,var,state)
	local output = {}
	for i,v in ipairs(data) do
		if v.pos == state then
			table.insert(output,i)
		end
	end
	return output
end

-- returns all the variables from an area within a table

function returnvar(data,var)
	local output = {}
	for i,v in ipairs(data) do
		table.insert(output,v[var])
	end
	return output
end

-- MENU CODE / MOUSE CONTROL CODE ( EXAMPLE CODE, every game needs different versions of this)
--[[

menutable = {
	{x=0,y=0,w=800,h=600,action={"skip1"}},
	
}

function nearestMenu(x,y)
	current = nearestestui(x,y,menutable)
	return current
end

function nearestestui(x,y,table)
	for i,v in ipairs(table) do
		if checkCollision(x,y,1,1,v.x,v.y,v.w,v.h) == true then
			id = i
		end
	end
	return id
end

function love.mousepressed(x,y,button)
	if button == "l" then
		mouseAction(x,y)
	end
end

function mouseAction(x,y)
	if gamestate == 0 then
		if menutable[nearestMenu(x,y)].action[1] == 0 then
			donothing = donothing
		elseif menutable[nearestMenu(x,y)].action[1] == "Singleplayer" then
			gamestate = 1
		elseif menutable[nearestMenu(x,y)].action[1] == "Coming Soon" then
			gamestate = 2
		elseif menutable[nearestMenu(x,y)].action[1] == "lastgmc" then
			if gmc - 1 < 1 then
				gmc = #gamemode
			elseif gmc - 1 > 0 then
				gmc = gmc - 1
			end
		elseif menutable[nearestMenu(x,y)].action[1] == "nextgmc" then
			if gmc + 1 > #gamemode then
				gmc = 1
			elseif gmc + 1 < #gamemode + 1 then
				gmc = gmc + 1
			end
		elseif menutable[nearestMenu(x,y)].action[1] == "quit" then
			love.event.push('q')
		end
	end
end
]]

-- BOX COLLISIONS

-- Collision detection function.
-- Checks if box1 and box2 overlap.
-- w and h mean width and height.
function checkCollision(box1x, box1y, box1w, box1h, box2x, box2y, box2w, box2h)
    if box1x > box2x + box2w - 1 or -- Is box1 on the right side of box2?
       box1y > box2y + box2h - 1 or -- Is box1 under box2?
       box2x > box1x + box1w - 1 or -- Is box2 on the right side of box1?
       box2y > box1y + box1h - 1    -- Is b2 under b1?
    then
        return false                -- No collision. Yay!
    else
        return true                 -- Yes collision. Ouch!
    end
end

-- SLIDER FUNCTION

-- moves A towards B over time
-- to use add the slider function to the update area
-- and define sliderinfo = {}

function slider(dt)
	if #sliderinfo > 0 then
		for i,v in ipairs(sliderinfo) do
			if v.a ~= v.b then
				if v.a > v.b then
					v.a = v.a - (v.speed*dt)
					if v.a < v.b then
						v.a = v.b
					end
				elseif v.a < v.b then
					v.a = v.a + (v.speed*dt)
					if v.a > v.b then
						v.a = v.b
					end
				end
			end
		end
	end
end

function zoom(dt)
	local zoom_dir = 0
	screenWidth =( 800 / 2 ) / (Z[1])
	screenHeight =( 600 / 2 ) / (Z[1])
	
	if Z[1] ~= Z[2] then
		if Z[1] > Z[2] then
			Z[1] = Z[1] - (1*dt)
			if Z[1] < Z[2] then
				Z[1] = Z[2]
			end
		elseif Z[1] < Z[2] then
			Z[1] = Z[1] + (1*dt)
			if Z[1] > Z[2] then
				Z[1] = Z[2]
			end
		end
	end
end

-- STATE/KEY CHANGE

-- looks through a table at any level and finds keys
-- then changes them to a "newkey"

function keyhunt(target,key,newkey)
	local temp = {}
	for i = 1,#target do 
		if target[i].state == key then
			table.insert(temp,i)
		end
	end
	
	for i = 1,#temp do
		target[temp[i]].state = newkey
	end	
end

function keytest(target,test)
	if target.I == answer then
		return 1
	end
end


-- test is the variable in table to test and answer is what it should equal
function keycull(target,key,newkey,test,answer)
	for i = 1,#table do
		if keytest(target[i],test,answer) == 1 then
			if target[i].state == key then
				target[i].state = newkey
			end
		end
	end
end

-- TEST IF SOMETHING IS ALREADY IN A TABLE
-- used to see if something should be added or not

function isintable(item,table)
	local output = false
	for i,v in ipairs(table) do
		if v == item then
			output = true
		end
	end
	return output
end

-- same as above but returns its position
		
function whereintable(item,table)
	local output = null
	for i,v in ipairs(table) do
		if v == item then
			output = i
		end
	end
	return output
end